using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; // point structure
using System.Xml;

namespace GenoProDOM
{
  /// <summary>
  /// Represent an GenoPro individual
  /// </summary>
  public class Individual
  {
    private string _Id;

    public string Id
    {
      get
      {
        return _Id;
      }
      set
      {
        _Id = value;
      }
    }


    public string Name;
    public bool IsMale = true;
    public Family ParentFamily;  // the family of your parent
    public Family ChildFamily;   // the family of your childs
    public DateTime Birth;
    public int XPosition;  // position in the X dimension
    public int YPosition;  // the Generation of the individual
 
    public int GenoMapIndex;

    public List<Individual> LinkedLeft = null; // other individual that must be moved if this individual is moved to the left
    public List<Individual> LinkedRight = null; // other individual that must be moved if this individual is moved to the right

    private const int PositionXMultiplication = 35;
    private const int PositionYMultiplication = -150;

    public Point Position
    {
      get
      {
        return new Point(XPosition * PositionXMultiplication, YPosition * PositionYMultiplication);
      }
    }

    public string PositionXml
    {
      get
      {
        Point position = Position;
        return string.Format("{0},{1}", position.X, position.Y);
      }
    }

    public void AddLinkedLeft(Individual ind)
    {
      if (LinkedLeft == null) {
        LinkedLeft = new List<Individual>();
      }
      LinkedLeft.Add(ind);
    }

    public void AddLinkedRight(Individual ind)
    {
      if (LinkedRight == null) {
        LinkedRight = new List<Individual>();
      }
      LinkedRight.Add(ind);
    }



    #region Property

    private bool _IsPlaced = false;

    public bool IsPlaced
    {
      get
      {
        return _IsPlaced;
      }
      set
      {
        _IsPlaced = value;
      }
    }


    #endregion


    public override string ToString()
    {
      return string.Format("{0} - {1} ({2},{3})", Name, Birth.Year, XPosition, YPosition);
    }

    public Individual(string id)
    {
      _Id = id;
    }

    public Individual(XmlNode individualNode)
    {
      _Id = individualNode.Attributes["ID"].Value;

      XmlNode nodeName = individualNode.SelectSingleNode(@"Name/Display/text()");
      if (nodeName != null) {
        Name = nodeName.Value;
      }

      XmlNode nodeBirth = individualNode.SelectSingleNode(@"Birth/Date/text()");
      if (nodeBirth != null) {
        if (!DateTime.TryParse(nodeBirth.Value, out Birth)) {
          // try for the year
          int year = 0;
          if (int.TryParse(nodeBirth.Value, out year)) {
            Birth = new DateTime(year, 1, 1);
          }
        }
      }

      XmlNode nodePosition = individualNode.SelectSingleNode(@"Position/text()");
      if (nodePosition != null) {
        string[] pos = nodePosition.Value.Split(',');
        XPosition = (int)Math.Floor(double.Parse(pos[0]) / PositionXMultiplication);
        YPosition = (int)Math.Floor(double.Parse(pos[1]) / PositionYMultiplication);
      }

      XmlNode nodeGender = individualNode.SelectSingleNode(@"Gender/text()");
      if (nodeGender != null) {
        IsMale = nodeGender.Value == "M";
      }
    }


    /// <summary>
    /// Find all individuals that will be affected if this individual need to be moved on the right
    /// </summary>
    /// <returns></returns>
    public void GetAllLinkedRight(List<Individual> lst)
    {
      if (LinkedRight != null) {
        foreach (Individual ind in LinkedRight) {
          if (!lst.Contains(ind)) {
            lst.Add(ind);
            ind.GetAllLinkedRight(lst);
          }
        }
      }

    } // GetAllLinkedRight

  } // class

} // namespace
